summaryrefslogtreecommitdiff
path: root/lib/stpncpy.c
diff options
context:
space:
mode:
Diffstat (limited to 'lib/stpncpy.c')
-rw-r--r--lib/stpncpy.c111
1 files changed, 57 insertions, 54 deletions
diff --git a/lib/stpncpy.c b/lib/stpncpy.c
index 79a0811..d1422a9 100644
--- a/lib/stpncpy.c
+++ b/lib/stpncpy.c
@@ -1,89 +1,92 @@
-/* Copyright (C) 1993, 1995, 1996 Free Software Foundation, Inc.
+/* Copyright (C) 1993, 1995-1997, 2002-2003, 2005-2007, 2009-2023 Free Software
+ * Foundation, Inc.
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3, or (at your option)
-any later version.
+ NOTE: The canonical source of this file is maintained with the GNU C Library.
+ Bugs can be reported to bug-glibc@gnu.org.
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
+ This file is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Lesser General Public License as
+ published by the Free Software Foundation; either version 2.1 of the
+ License, or (at your option) any later version.
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+ This file is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ GNU Lesser General Public License for more details.
+
+ You should have received a copy of the GNU Lesser General Public License
+ along with this program. If not, see <https://www.gnu.org/licenses/>. */
/* This is almost copied from strncpy.c, written by Torbjorn Granlund. */
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
-#include <sys/types.h>
+/* Specification. */
+#include <string.h>
+#ifndef weak_alias
+# define __stpncpy stpncpy
+#endif
-/* Copy no more than N characters of SRC to DEST, returning the address of
- the last character written into DEST. */
+/* Copy no more than N bytes of SRC to DST, returning a pointer past the
+ last non-NUL byte written into DST. */
char *
-stpncpy (dest, src, n)
- char *dest;
- const char *src;
- size_t n;
+(__stpncpy) (char *dest, const char *src, size_t n)
{
char c;
char *s = dest;
- --dest;
-
if (n >= 4)
{
size_t n4 = n >> 2;
for (;;)
- {
- c = *src++;
- *++dest = c;
- if (c == '\0')
- break;
- c = *src++;
- *++dest = c;
- if (c == '\0')
- break;
- c = *src++;
- *++dest = c;
- if (c == '\0')
- break;
- c = *src++;
- *++dest = c;
- if (c == '\0')
- break;
- if (--n4 == 0)
- goto last_chars;
- }
- n = n - (dest - s) - 1;
- if (n == 0)
- return dest + 1;
+ {
+ c = *src++;
+ *dest++ = c;
+ if (c == '\0')
+ break;
+ c = *src++;
+ *dest++ = c;
+ if (c == '\0')
+ break;
+ c = *src++;
+ *dest++ = c;
+ if (c == '\0')
+ break;
+ c = *src++;
+ *dest++ = c;
+ if (c == '\0')
+ break;
+ if (--n4 == 0)
+ goto last_chars;
+ }
+ n -= dest - s;
goto zero_fill;
}
last_chars:
n &= 3;
if (n == 0)
- return dest + 1;
+ return dest;
- do
+ for (;;)
{
c = *src++;
- *++dest = c;
- if (--n == 0)
- return dest + 1;
+ --n;
+ *dest++ = c;
+ if (c == '\0')
+ break;
+ if (n == 0)
+ return dest;
}
- while (c != '\0');
zero_fill:
while (n-- > 0)
dest[n] = '\0';
- return dest;
+ return dest - 1;
}
+#ifdef weak_alias
+weak_alias (__stpncpy, stpncpy)
+#endif